#!/bin/bash
########################################################################
## Script    : cert-mkconf.sh 
## Name      : openssl の拡張設定ファイル生成スクリプト
## Version   : 0.0.1
## Copyright : 2018-2019  Nomura Kei
## License   : BSD-2-Clause
## Usage:
## |使用法) cert-mkconf.sh [オプション] CA証明書のURL
## |
## |   openssl の拡張設定ファイルを生成します。
## |
## |[オプション]
## |  --cps arg      CPS の URL を指定します。
## |  --cps-oid arg  CPS の OID を指定します。
## |  --crl arg      CRL を配置する URL を指定します。
## |  --ocsp arg     OCSP の URL を指定します。
## |  -h,--help      使用法を表示します。
## |  -v,--version   バージョンを表示します。
## |
## |[使用例]
## |mksslextconf.sh --crl http://ehobby.jp/ca/latestcrl.crl \
## |                --ocsp http://ehobby.jp/ca/ocsp         \
## |                 http://ehobby.jp/ca/ca.crt
## |
## |[参考]
## |OCSP サーバーの起動方法
## |oppenssl ocsp -index index.txt -CA rootca.pem \
## |    -rsigner rootca.pem -rkey private/prikey.pem -port 8080
## |
##
########################################################################
SCRIPT_FILE=${0}
SCRIPT_DIR=`dirname ${SCRIPT_FILE}`
DIR=${SCRIPT_DIR}/../conf

CA=
CPS=
CPSOID=
CRL=
OCSP=

 
########################################################################
##
## ライブラリのロード
##
[ -f "${SCRIPT_DIR}/functions.sh" ] && . ${SCRIPT_DIR}/functions.sh
 

# ======================================================================
#  引数をチェックします。
#  引数に問題がある場合、エラーを表示して終了します。
# ======================================================================
function checkArgs() {
	# CA 有無チェック
	if [ "${CA}" = "" ]; then
		echo "CAのURLを指定してください。"
		usage
		exit 1
	fi
	if [ "${CRL}" = "" ] && [ "${OCSP}" = "" ]; then
		echo "CRL または、OCSP を指定してください。"
		usage
		exit 1
	fi
}


# ======================================================================
#  設定ファイルをバックアップします。
# ======================================================================
function backup_cnf() {
	BACKUP_FILES="${DIR}/openssl_ca.cnf ${DIR}/openssl_server.cnf ${DIR}/openssl_client.cnf"
	for f in $BACKUP_FILES; do
		if [ -f $f ]; then
			mv -f $f $f.old
		fi
	done
}


# ======================================================================
#  共通の下記情報を出力します。
#  ・CRL 配布ポイント
#  ・認証ポリシー
#  ・権限情報へのアクセス
#  
#  @param $1 認証ポリシーのポリシー識別
#            P1 : ドメイン検証 (CA, サーバー証明書などで利用される)
#            P2 : 組織検証
#            P3 : 個別検証
# ======================================================================
function print_cnf_cominfo() {
	POLICIES_OPT=$1

	# CRL 配布ポイント出力
	if [ "${CRL}" != "" ]; then
		echo "crlDistributionPoints   = URI: ${CRL}"
	fi 

	# 認証ポリシー出力
	if [ "${CPS}" != "" ] && [ "${CPSOID}" != "" ]; then
		echo "certificatePolicies     = @policies, @cps" 
	else
		echo "certificatePolicies     = @policies" 
	fi
	
	# 権限情報へのアクセス ～ 認証ポリシーセクション出力
cat <<EOF | sed "s/^#-${POLICIES_OPT} //"
authorityInfoAccess     = @info_access

[ policies ]
#-P1 policyIdentifier       = 2.23.140.1.2.1    # ドメイン検証
#-P2 policyIdentifier       = 2.23.140.1.2.2    # 組織検証
#-P3 policyIdentifier       = 2.23.140.1.2.3    # 個別検証

EOF

	# 認証ポリシー (CPS 情報) 出力
	if [ "${CPS}" != "" ] && [ "${CPSOID}" != "" ]; then
		echo "[ cps ]"
		echo "policyIdentifier        = ${CPSOID}"
		echo "CPS.1                   = ${CPS}"
		echo ""
	fi

	
	# 権限情報へのアクセスセクション出力
	echo "[ info_access ]"
	echo "caIssuers;URI.0         = ${CA}"
	if [ "${OCSP}" != "" ]; then
		echo "OCSP;URI.0              = ${OCSP}"
	fi
	echo ""
}



# ======================================================================
#  中間認証局 CA証明書 署名用 の設定内容を出力します。
# ======================================================================
function print_cnf_ca() {
cat <<EOF
####################################################################
##
## 中間認証局 CA証明書 署名用
##
[ sign_ca ]
#
# subjectKeyIdentifier
#   サブジェクトキー識別子。hash を指定する。
#   ※16進数文字列してもあるが、非推奨
subjectKeyIdentifier    = hash
#
#
# authorityKeyIdentifier
#   keyid  : サブジェクトキー識別子を親証明書からコピーする。
#   issuer : 発行者証明書から発行者とシリアル番号をコピーする。
#   keyid:always, issuer:always のように、always を指定すると、
#   常にその値のコピーを強制し、コピーできない場合はエラーとなる。
authorityKeyIdentifier  = keyid:always,issuer
#
#
# extendedKeyUsage
#   証明書公開鍵を使用できる目的を示す用法リスト
#   次の値を指定可能である。
#   - serverAuth      : SSL/TLS Webサーバー認証
#   - clientAuth      : SSL/TLS Webクライアント認証
#   - codeSigning     : コード署名
#   - emailProtection : 電子メール保護 (S/MIMEなど)
#   - timeStamping    : 信頼できるタイムスタンプ
#   - msSGC           : Microsoft Server Gated Crypto
#   - nsSGC           : Netscape Server Gated Crypto
#   - msSmartcardLogin: Microsoft Smartcardlogin
extendedKeyUsage        = serverAuth,clientAuth,emailProtection,codeSigning
#
# 
# CA証明書か否かを表す。
# CA証明書の場合は、critical,CA:TRUE を指定する。
basicConstraints        = critical,CA:TRUE,pathlen: 0
keyUsage                = critical,digitalSignature,cRLSign,keyCertSign
EOF

# 共通情報出力
print_cnf_cominfo "P1"
}


# ======================================================================
#  サーバー証明書署名用の設定内容を出力します。
# ======================================================================
function print_cnf_server() {
cat <<EOF
####################################################################
##
## サーバー証明書 署名用
##
[ sign_server ]
#
# subjectKeyIdentifier
#   サブジェクトキー識別子。hash を指定する。
#   ※16進数文字列してもあるが、非推奨
subjectKeyIdentifier    = hash
#
#
# authorityKeyIdentifier
#   keyid  : サブジェクトキー識別子を親証明書からコピーする。
#   issuer : 発行者証明書から発行者とシリアル番号をコピーする。
#   keyid:always, issuer:always のように、always を指定すると、
#   常にその値のコピーを強制し、コピーできない場合はエラーとなる。
authorityKeyIdentifier  = keyid,issuer
#
#
# subjectAltName
#   代替名。次のような値を指定可能。
#   ・DNS
#   ・email
#   ・URI
#   ・RID
#   ・IP
#   ・dirName
#   ・otherName
#   サーバー証明書の場合、DNS の指定が必要であり、Chrome では、
#   本指定が無ければ ERR_CERT_COMMON_NAME_INVALID のエラーがっ製する。
#
#   カンマ区切りあるいは、セクションを分けて複数指定することが可能である。
#
#   例1) カンマ区切りによる指定
#   subjectAltName=email:sample@ehobby.jp,RID:1.2.3.4
#
#   例2) 別セッションにて指定
#   subjectAltName=@alt_names
subjectAltName=@alt_names
#
#
# extendedKeyUsage
#   証明書公開鍵を使用できる目的を示す用法リスト
#   次の値を指定可能である。
#   - serverAuth      : SSL/TLS Webサーバー認証
#   - clientAuth      : SSL/TLS Webクライアント認証
#   - codeSigning     : コード署名
#   - emailProtection : 電子メール保護 (S/MIMEなど)
#   - timeStamping    : 信頼できるタイムスタンプ
#   - msSGC           : Microsoft Server Gated Crypto
#   - nsSGC           : Netscape Server Gated Crypto
#   - msSmartcardLogin: Microsoft Smartcardlogin
extendedKeyUsage        = serverAuth,clientAuth
#
# 
# CA証明書か否かを表す。
# CA証明書の場合は、critical,CA:TRUE を指定する。
basicConstraints        = critical,CA:FALSE
keyUsage                = critical,digitalSignature,keyEncipherment
EOF

# 共通情報出力
print_cnf_cominfo "P1"
}


# ======================================================================
#  クライアント証明書署名用の設定内容を出力します。
# ======================================================================
function print_cnf_client() {
cat <<EOF
####################################################################
##
## クライアント証明書 署名用
##
[ sign_client ]
#
# subjectKeyIdentifier
#   サブジェクトキー識別子。hash を指定する。
#   ※16進数文字列してもあるが、非推奨
subjectKeyIdentifier    = hash
#
#
# authorityKeyIdentifier
#   keyid  : サブジェクトキー識別子を親証明書からコピーする。
#   issuer : 発行者証明書から発行者とシリアル番号をコピーする。
#   keyid:always, issuer:always のように、always を指定すると、
#   常にその値のコピーを強制し、コピーできない場合はエラーとなる。
authorityKeyIdentifier  = keyid,issuer
#
#
# subjectAltName
#   代替名。次のような値を指定可能。
#   ・DNS
#   ・email
#   ・URI
#   ・RID
#   ・IP
#   ・dirName
#   ・otherName
#   サーバー証明書の場合、DNS の指定が必要であり、Chrome では、
#   本指定が無ければ ERR_CERT_COMMON_NAME_INVALID のエラーがっ製する。
#
#   カンマ区切りあるいは、セクションを分けて複数指定することが可能である。
#
#   例1) カンマ区切りによる指定
#   subjectAltName=email:sample@ehobby.jp,RID:1.2.3.4
#
#   例2) 別セッションにて指定
#   subjectAltName=@alt_names
#subjectAltName=@alt_names
#
#
# extendedKeyUsage
#   証明書公開鍵を使用できる目的を示す用法リスト
#   次の値を指定可能である。
#   - serverAuth      : SSL/TLS Webサーバー認証
#   - clientAuth      : SSL/TLS Webクライアント認証
#   - codeSigning     : コード署名
#   - emailProtection : 電子メール保護 (S/MIMEなど)
#   - timeStamping    : 信頼できるタイムスタンプ
#   - msSGC           : Microsoft Server Gated Crypto
#   - nsSGC           : Netscape Server Gated Crypto
#   - msSmartcardLogin: Microsoft Smartcardlogin
extendedKeyUsage        = clientAuth
#
# 
# CA証明書か否かを表す。
# CA証明書の場合は、critical,CA:TRUE を指定する。
basicConstraints        = critical,CA:FALSE
keyUsage                = critical,digitalSignature,keyEncipherment
EOF

# 共通情報出力
print_cnf_cominfo "P3"
}

# ======================================================================
#  openssl の拡張設定ファイルを生成します。
#  具体的には、次の処理を実施します。
#  (1) 引数チェック
#  (2) 設定ファイルのバックアップ
#  (3) 中間認証局用の CA署名用の拡張設定ファイル生成
#  (4) サーバー証明書署名用の拡張設定ファイル生成
#  (5) クライアント証明書署名用の拡張設定ファイル生成
#
# ======================================================================
function cert_mkcnf() {
	checkArgs											# (1)
	backup_cnf											# (2)
	print_cnf_ca		> ${DIR}/openssl_ca.cnf			# (3)
	print_cnf_server	> ${DIR}/openssl_server.cnf		# (4)
	print_cnf_client	> ${DIR}/openssl_client.cnf		# (5)
	echo "設定ファイルを出力しました。"
}


########################################################################
##
## メイン処理
##
for OPT in "$@"; do
	case "${OPT}" in
		'-h'|'--help')		usage;								exit 1	;;
		'-v'|'--version')	version;							exit 1	;;
		'--cps')			checkOptionArgs "$2"; CPS="$2";		shift 2	;;
		'--cps-oid')		checkOptionArgs "$2"; CPSOID="$2";	shift 2	;;
		'--crl')			checkOptionArgs "$2"; CRL="$2";		shift 2	;;
		'--ocsp')			checkOptionArgs "$2"; OCSP="$2";	shift 2	;;
		-*)
			usage
			exit 1
			;;
		*)
			if [[ ! -z "$1" ]] && [[ ! "$1" =~ ^-+ ]]; then
				CA=("$1")
				shift 1
			fi
			;;
	esac
done


cert_mkcnf

