#!/bin/bash
########################################################################
## Script    : cert-mkcsr.sh
## Name      : CSR 生成スクリプト
## Version   : 0.0.1
## Copyright : 2018-2019  Nomura Kei
## License   : BSD-2-Clause
## Usage:
## |使用法) cert-mkcsr.sh [オプション] [subject]
## |
## |  CSR を作成します。
## |  秘密鍵が未生成の場合、秘密鍵を合わせて生成します。
## |
## |subject          次に記す必要な値を続けて記述します。
## |                   /C=Country               [国名]
## |                   /ST=State or Province    [都道府県名]
## |                   /L=Locality              [市町村名]
## |                   /O=Organization          [組織名]
## |                   /OU=Organizational Unit  [部門名]
## |                   /CN=Common Name          [コモンネーム]
## |                 省略された場合、対話形式で入力します。
## |  例1) CA証明書
## |    "/C=JP/O=Ehobby/CN=Ehobby Root CA"
## |  例2) サーバー証明書
## |    "/C=JP/ST=Osaka/L=Osaka/O=Ehobby/OU=Ehobby Web/CN=ehobby.jp"
## |    ※ドメイン名を CN に指定します。
## |  例3) クライアント証明書
## |    "/C=JP/ST=Osaka/L=Osaka/O=Ehobby/OU=Ehobby Web/CN=Ehobby"
## |    ※通常, 会社名, 部門名, 個人名などを CN に設定します。
## |                 
## |[オプション]
## |  --days arg    有効日数を指定します。
## |                 (default: --days 730)
## |  --out arg      CSR 出力先を指定します。
## |                 (default: --out request.csr)
## |  --hash arg     署名ハッシュアルゴリズムを指定します。
## |                 ハッシュアルゴリズムとして次の値を指定可能です。
## |                   md4, md5, ripemd160, whirlpool
## |                   sha, sha1, sha224, sha256, sha384, sha512
## |                 (default: --hash sha256
## |                 (default: --newkey rsa:2048)
## |  --key arg      秘密鍵を指定します。
## |                 (default: --key private/privkey.pem)
## |  --bits arg     [秘密鍵生成時] ビット数を指定します。
## |  --crypt arg    [秘密鍵生成時] 暗号化アルゴリズムを指定します。
## |                 暗号化アルゴリズムとして次の値を指定可能です。
## |                   none, des, des3, idea,
## |                   aes128, aes192, aes256,
## |                   camellia128, camellia192, camellia256
## |                 (default: --crypt aes256)
## |  -h,--help      使用法を表示します。
## |  -v,--version   バージョンを表示します。
## |
##
########################################################################
SCRIPT_FILE=${0}
SCRIPT_DIR=`dirname ${SCRIPT_FILE}`
DIR=${SCRIPT_DIR}/..
CONF=${SCRIPT_DIR}/../conf/openssl.cnf
CRYPT=aes256
DAYS=730
OUT=${DIR}/request.csr
HASH=sha256
KEY=${SCRIPT_DIR}/../private/privkey.pem
BITS=2048
SUBJ=


########################################################################
##
## ライブラリのロード
##
[ -f "${SCRIPT_DIR}/functions.sh" ] && . ${SCRIPT_DIR}/functions.sh


# ======================================================================
#  引数をチェックします。
#  引数に問題がある場合、エラーを表示して終了します。
# ======================================================================
function checkArgs() {
	# 1. CRYPT  チェック => 不要
	# 2. DAYS チェック
	if [ "${DAYS}" = "" ]; then
		echo "--days の指定に誤りがあります。"
		exit 1
	fi
	# 3. OUT チェック
	if [ "${OUT}" = "" ]; then
		echo "--out の指定に誤りがあります。"
		exit 1
	fi
	# 4. HASH チェック
	if [ "${HASH}" = "" ]; then
		echo "--hash の指定に誤りがあります。"
		exit 1
	fi
	# 5. NEWKEY チェック
	if [ "${BITS}" = "" ]; then
		echo "--bits の指定に誤りがあります。"
		exit 1
	fi
}



# ======================================================================
#  CSR を生成します。
# ======================================================================
function cert_mkcsr() {
	# 1. 引数チェック
	checkArgs

	
	# 2. 秘密鍵がなければ生成
	FULLPATH_KEY=`getAbsolutePath ${KEY}`
	if [ ! -f ${FULLPATH_KEY} ]; then
		confirm "${FULLPATH_KEY} を生成します。" 1
		if [ $? -eq 0 ]; then
			exit 1
		fi
		${SCRIPT_DIR}/cert-mkkey.sh --bits ${BITS} --crypt ${CRYPT} --out ${KEY}
		if [ $? -ne 0 ]; then
			exit 1
		fi
	fi

	# 3. 古いファイルをバックアップ
	if [ -f ${OUT} ]; then
		mv -f ${OUT} ${OUT}.old
	fi

	# 4. CSR 生成
	OPT_SUBJ="-subj ${SUBJ}"
	if [ "${SUBJ}" = "" ]; then
		OPT_SUBJ=
	fi
		
	openssl req -new			\
		-config ${CONF}			\
		-key    ${FULLPATH_KEY}	\
		-days   ${DAYS}			\
		-${HASH}				\
		${OPT_SUBJ}				\
		-out "${OUT}"

}



########################################################################
##
## メイン処理
##
for OPT in "$@"; do
	case "${OPT}" in
		'-h'|'--help')		usage;								exit 1	;;
		'-v'|'--version')	version;							exit 1	;;
		'--bits')			checkOptionArgs "$2"; BITS="$2";	shift 2	;;
		'--crypt')			checkOptionArgs "$2"; CRYPT="$2";	shift 2	;;
		'--days')			checkOptionArgs "$2"; DAYS="$2";	shift 2	;;
		'--out')			checkOptionArgs "$2"; OUT="$2";		shift 2	;;
		'--hash')			checkOptionArgs "$2"; HASH="$2";	shift 2	;;
		-*)
			usage
			exit 1
			;;
		*)
			if [[ ! -z "$1" ]] && [[ ! "$1" =~ ^-+ ]]; then
				SUBJ=("$1")
				shift 1
			fi
			;;
	esac
done

cert_mkcsr

