#!/bin/bash
########################################################################
## Script    : cert-mkkey.sh
## Name      : 秘密鍵作成スクリプト
## Version   : 0.0.1
## Copyright : 2018-2019  Nomura Kei
## License   : BSD-2-Clause
## Usage:
## |使用法) cert-mkkey.sh [オプション]
## |
## |  秘密鍵を作成します。
## |
## |[オプション]
## |  --bits arg     ビット数
## |  --crypt arg    暗号化アルゴリズムを指定します。
## |                 暗号化アルゴリズムとして次の値を指定可能です。
## |                   none, des, des3, idea,
## |                   aes128, aes192, aes256,
## |                   camellia128, camellia192, camellia256
## |                 (default: --crypt aes256)
## |  --out arg      秘密鍵出力先を指定します。
## |                 (default: --out private/privkey.pem)
## |  -h,--help      使用法を表示します。
## |  -v,--version   バージョンを表示します。
## |
##
########################################################################
SCRIPT_FILE=${0}
SCRIPT_DIR=`dirname ${SCRIPT_FILE}`
BITS=2048
DIR=${SCRIPT_DIR}/../private
CRYPT=aes256
OUT=${DIR}/privkey.pem


########################################################################
##
## ライブラリのロード
##
[ -f "${SCRIPT_DIR}/functions.sh" ] && . ${SCRIPT_DIR}/functions.sh


# ======================================================================
#  引数をチェックします。
#  引数に問題がある場合、エラーを表示して終了します。
# ======================================================================
function checkArgs() {
	# 1. BITS チェック
	if [ "${BITS}" = "" ]; then
		echo "--bits の指定に誤りがあります。"
		exit 1
	fi
	# 2. CRYPT  チェック
	if [ "${CRYPT}" = "none" ]; then
		CRYPT=
	else
		CRYPT=-${CRYPT}
	fi
	# 3. OUT チェック
	if [ "${OUT}" = "" ]; then
		echo "--out の指定に誤りがあります。"
		exit 1
	fi
}



# ======================================================================
#  秘密鍵を生成します。
#
#  @param $1 秘密鍵出力先
#  @param $2 暗号化アルゴリズム
# ======================================================================
function cert_mkkey() {
	checkArgs
	FULLPATH=`getAbsolutePath ${OUT}`
	overwrite ${FULLPATH}

	openssl genrsa ${CRYPT} -out ${FULLPATH} ${BITS}
	if [ $? -eq 0 ]; then
		chmod 600 ${FULLPATH}
		echo "秘密鍵(${FULLPATH})を生成しました。"
	else
		echo "秘密鍵(${FULLPATH})の生成に失敗しました。"
		exit 1
	fi
}



########################################################################
##
## メイン処理
##
for OPT in "$@"; do
	case "${OPT}" in
		'-h'|'--help')		usage;								exit 1	;;
		'-v'|'--version')	version;							exit 1	;;
		'--bits')			checkOptionArgs "$2"; BITS="$2";	shift 2	;;
		'--crypt')			checkOptionArgs "$2"; CRYPT="$2";	shift 2	;;
		'--out')			checkOptionArgs "$2"; OUT="$2";		shift 2	;;
		-*)
			usage
			exit 1
			;;
		*)
			if [[ ! -z "$1" ]] && [[ ! "$1" =~ ^-+ ]]; then
				usage
				exit 1
			fi
			;;
	esac
done

cert_mkkey

