#!/bin/bash
########################################################################
## Script    : cert-sign.sh
## Name      : 署名スクリプト
## Version   : 0.0.1
## Copyright : 2018-2019  Nomura Kei
## License   : BSD-2-Clause
## Usage:
## |使用法) cert-sign.sh [オプション] csrfile
## |
## |  CSR ファイルに署名します。
## |
## |csrfile         署名するCSRファイル
## |                 
## |[オプション]
## |  --days arg    有効日数を指定します。
## |                 (default: --days 730)
## |  --dns1 arg    [サーバー証明書] ドメイン名1
## |  --dns2 arg    [サーバー証明書] ドメイン名2
## |  --dns3 arg    [サーバー証明書] ドメイン名3
## |  --md arg      メッセージダイジェスト
## |                メッセージダイジェストとして次の値を指定可能です。
## |                  md4, md5, ripemd160, whirlpool
## |                  sha, sha1, sha224, sha256, sha384, sha512
## |                (default: --md sha256)
## |  --type arg    署名する証明書の種類を指定します。
## |                  rootca : ルート認証局用証明書(自己署名)
## |                  ca     : 中間認証局用証明書
## |                  server : サーバー証明書
## |                  client : クライアント証明書
## |                (default: --type server)
## |  -h,--help     使用法を表示します。
## |  -v,--version  バージョンを表示します。
## |
##
########################################################################
SCRIPT_FILE=${0}
SCRIPT_DIR=`dirname ${SCRIPT_FILE}`
DIR=${SCRIPT_DIR}/..
CONF=${SCRIPT_DIR}/../conf/openssl.cnf
CONF_CA=${SCRIPT_DIR}/../conf/openssl_ca.cnf
CONF_SERVER=${SCRIPT_DIR}/../conf/openssl_server.cnf
CONF_CLIENT=${SCRIPT_DIR}/../conf/openssl_client.cnf
CONF_EXT=${SCRIPT_DIR}/../conf/forsign.ext
CSR=
DAYS=730
DNS1=
DNS2=
DNS3=
KEY=${SCRIPT_DIR}/../private/privkey.pem
MD=sha256
TYPE=server


########################################################################
##
## ライブラリのロード
##
[ -f "${SCRIPT_DIR}/functions.sh" ] && . ${SCRIPT_DIR}/functions.sh


# ======================================================================
#  引数をチェックします。
#  引数に問題がある場合、エラーを表示して終了します。
# ======================================================================
function checkArgs() {
	# 1. CSR チェック
	if [ "${CSR}" = "" ] || [ ! -f ${CSR} ]; then
		echo "CSR ファイルを指定してください。"
		exit 1
	fi

	# 2. DAYS チェック
	if [ "${DAYS}" = "" ]; then
		echo "--days の指定に誤りがあります。"
		exit 1
	fi

	# 3. MD チェック
	if [ "${MD}" = "" ]; then
		echo "--md の指定に誤りがあります。"
		exit 1
	fi

	# 4. TYPE チェック
	case "${TYPE}" in
	'rootca')	;;
	'ca')		;;
	'server')
		if [ "${DNS1}" = "" ]; then
			echo "サーバー証明書の署名には、ドメイン名指定が必須です。"
			exit
		fi
		;;
	'client')	;;
	*)
		echo "--type の指定に誤りがあります。"
		exit 1
		;;
	esac
}



# ======================================================================
#  一時拡張設定ファイルを生成します。
# ======================================================================
function mkextfile() {
	case "${TYPE}" in
	'rootca')
		;;
	'ca')
		cat ${CONF_CA}					> ${CONF_EXT}
		;;
	'server')
		cat ${CONF_SERVER}				>  ${CONF_EXT}
		echo ""							>> ${CONF_EXT}
		echo "[ alt_names ]"			>> ${CONF_EXT}
		echo "DNS.1		= ${DNS1}"		>> ${CONF_EXT}
		if [ "${DNS2}" != "" ]; then
			echo "DNS.2		= ${DNS2}"	>> ${CONF_EXT}
		fi
		if [ "${DNS3}" != "" ]; then
			echo "DNS.3		= ${DNS3}"	>> ${CONF_EXT}
		fi
		;;
	'client')
		cat ${CONF_CLIENT}				> ${CONF_EXT}
		;;
	*)
		;;
	esac
}



# ======================================================================
#  署名します。
# ======================================================================
function cert_sign() {
	# 1. 引数チェック
	checkArgs

	# 2. 拡張設定ファイル生成 (サーバー証明書の場合のみ)
	mkextfile

	BASENAME=`echo "${CSR}" | sed "s/^\(.*\)\..*$/\1/"`
	PEM="${BASENAME}_signed.pem"
	CRT="${BASENAME}_signed.crt"
	RETCODE=

	FULL_CSR=`getAbsolutePath ${CSR}`
	FULL_ROOTCSR=`getAbsolutePath ${DIR}/request.csr`
	if [ "${FULL_CSR}" = "${FULL_ROOTCSR}" ]	\
	&& [ "${TYPE}" = "rootca" ]; then
		PEM=${DIR}/cacert.pem
		CRT=${DIR}/cacert.crt
	fi


	case "${TYPE}" in
	'rootca')
		openssl x509 -req				\
			-${MD}						\
			-extfile ${CONF}			\
			-extensions v3_ca			\
			-days ${DAYS}				\
			-out ${PEM}					\
			-in ${CSR}					\
			-signkey ${KEY}
		RETCODE=$?
		;;
	*)
		openssl ca -config ${CONF}		\
			-extfile ${CONF_EXT}		\
			-extensions sign_${TYPE}	\
			-md ${MD}					\
			-policy policy_match		\
			-days ${DAYS}				\
			-out ${PEM}					\
			-infiles ${CSR}
		RETCODE=$?
		;;
	esac

	if [ ${RETCODE} -ne 0 ]; then
		exit 1
	fi

	# 2.  CRT ファイル生成
	openssl x509 -inform pem -in ${PEM} \
		-outform der -out ${CRT}

	echo "次のファイルを生成しました。"
	echo "  ${PEM}"
	echo "  ${CRT}"

}



########################################################################
##
## メイン処理
##
for OPT in "$@"; do
	case "${OPT}" in
		'-h'|'--help')		usage;								exit 1	;;
		'-v'|'--version')	version;							exit 1	;;
		'--days')			checkOptionArgs "$2"; DAYS="$2";	shift 2	;;
		'--dns1')			checkOptionArgs "$2"; DNS1="$2";	shift 2	;;
		'--dns2')			checkOptionArgs "$2"; DNS2="$2";	shift 2	;;
		'--dns3')			checkOptionArgs "$2"; DNS3="$2";	shift 2	;;
		'--md')				checkOptionArgs "$2"; MD="$2";		shift 2	;;
		'--type')			checkOptionArgs "$2"; TYPE="$2";	shift 2	;;
		-*)
			usage
			exit 1
			;;
		*)
			if [[ ! -z "$1" ]] && [[ ! "$1" =~ ^-+ ]]; then
				CSR=("$1")
				shift 1
			fi
			;;
	esac
done

cert_sign

